## $Id: documents.py 12018 2014-11-21 08:07:15Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Customer document components.
"""
import grok
from zope.component import queryUtility, getUtility
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.customers.interfaces import (
    ICustomerDocumentsContainer, ICustomerNavigation, ICustomerDocument,
    ICustomersUtils)
from waeup.ikoba.documents import DocumentsContainer, Document
from waeup.ikoba.documents.interfaces import IDocumentsUtils
from waeup.ikoba.utils.helpers import attrs_to_fields

class CustomerDocumentsContainer(DocumentsContainer):
    """This is a container for customer documents.
    """
    grok.implements(ICustomerDocumentsContainer, ICustomerNavigation)
    grok.provides(ICustomerDocumentsContainer)

    def __init__(self):
        super(CustomerDocumentsContainer, self).__init__()
        return

    @property
    def customer(self):
        return self.__parent__

    def writeLogMessage(self, view, message):
        return self.__parent__.writeLogMessage(view, message)

CustomerDocumentsContainer = attrs_to_fields(CustomerDocumentsContainer)

class CustomerDocument(Document):
    """This is an document.
    """
    grok.implements(ICustomerDocument, ICustomerNavigation)
    grok.provides(ICustomerDocument)

    def __init__(self):
        super(CustomerDocument, self).__init__()
        return

    @property
    def customer(self):
        try:
            return self.__parent__.__parent__
        except AttributeError:
            return None

    def writeLogMessage(self, view, message):
        return self.__parent__.__parent__.writeLogMessage(view, message)

    @property
    def is_editable(self):
        try:
            # Customer must be approved
            cond1 = self.customer.state in getUtility(
                ICustomersUtils).DOCMANAGE_STATES
            # Document must be in state created
            cond2 = self.state in getUtility(
                IDocumentsUtils).DOCMANAGE_STATES
            if not (cond1 and cond2):
                return False
        except AttributeError:
            pass
        return True

CustomerDocument = attrs_to_fields(CustomerDocument)


# Customer documents must be importable. So we might need a factory.
class CustomerDocumentFactory(grok.GlobalUtility):
    """A factory for customer documents.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.CustomerDocument')
    title = u"Create a new document.",
    description = u"This factory instantiates new document instances."

    def __call__(self, *args, **kw):
        return CustomerDocument(*args, **kw)

    def getInterfaces(self):
        return implementedBy(CustomerDocument)
